@@ -8,5 +8,4 @@ Licensed under GNU Lesser General Public License v3.0
 from dlclive.version import __version__, VERSION
 from dlclive.dlclive import DLCLive
 from dlclive.processor import Processor
-from dlclive.bench import benchmark_model_by_size
-from dlclive.benchmark import benchmark, benchmark_videos
+from dlclive.benchmark import benchmark, benchmark_videos, download_benchmarking_data
